@@ -4,9 +4,12 @@
  * @author Duzy Chan <code@duzy.info>
  * @author Jonathan Thomas <jonathan@openshot.org> *
  *
- * @section LICENSE
+ * @ref License
+ */
+
+/* LICENSE
  *
- * Copyright (c) 2008-2014 OpenShot Studios, LLC
+ * Copyright (c) 2008-2019 OpenShot Studios, LLC
  * <http://www.openshotstudios.com/>. This file is part of
  * OpenShot Library (libopenshot), an open-source project dedicated to
  * delivering high quality video editing and animation solutions to the
@@ -26,7 +29,10 @@
  * along with OpenShot Library. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "../../include/Qt/AudioPlaybackThread.h"
+#include "AudioPlaybackThread.h"
+
+#include <thread>    // for std::this_thread::sleep_for
+#include <chrono>    // for std::chrono::milliseconds
 
 namespace openshot
 {
@@ -35,18 +41,44 @@ namespace openshot
 	AudioDeviceManagerSingleton *AudioDeviceManagerSingleton::m_pInstance = NULL;
 
 	// Create or Get an instance of the device manager singleton
-	AudioDeviceManagerSingleton *AudioDeviceManagerSingleton::Instance(int numChannels)
+	AudioDeviceManagerSingleton *AudioDeviceManagerSingleton::Instance()
 	{
 		if (!m_pInstance) {
 			// Create the actual instance of device manager only once
 			m_pInstance = new AudioDeviceManagerSingleton;
 
+			// Get preferred audio device name (if any)
+			juce::String preferred_audio_device = juce::String(Settings::Instance()->PLAYBACK_AUDIO_DEVICE_NAME.c_str());
+
 			// Initialize audio device only 1 time
-            m_pInstance->audioDeviceManager.initialise (
+			juce::String audio_error = m_pInstance->audioDeviceManager.initialise (
 					0, /* number of input channels */
-                    numChannels, /* number of output channels */
+					2, /* number of output channels */
 					0, /* no XML settings.. */
-					true  /* select default device on failure */);
+					true,  /* select default device on failure */
+					preferred_audio_device /* preferredDefaultDeviceName */);
+
+			// Persist any errors detected
+			if (audio_error.isNotEmpty()) {
+				m_pInstance->initialise_error = audio_error.toRawUTF8();
+			} else {
+				m_pInstance->initialise_error = "";
+			}
+
+			// Get all audio device names
+			for (int i = 0; i < m_pInstance->audioDeviceManager.getAvailableDeviceTypes().size(); ++i)
+			{
+				const AudioIODeviceType* t = m_pInstance->audioDeviceManager.getAvailableDeviceTypes()[i];
+				const juce::StringArray deviceNames = t->getDeviceNames ();
+
+				for (int j = 0; j < deviceNames.size (); ++j )
+				{
+					juce::String deviceName = deviceNames[j];
+					juce::String typeName = t->getTypeName();
+					openshot::AudioDeviceInfo deviceInfo = {deviceName.toRawUTF8(), typeName.toRawUTF8()};
+					m_pInstance->audio_device_names.push_back(deviceInfo);
+				}
+			}
 		}
 
 		return m_pInstance;
@@ -63,7 +95,7 @@ namespace openshot
 
     // Constructor
     AudioPlaybackThread::AudioPlaybackThread()
-	: Thread("audio-playback")
+	: juce::Thread("audio-playback")
 	, player()
 	, transport()
 	, mixer()
@@ -82,7 +114,7 @@ namespace openshot
     }
 
     // Set the reader object
-    void AudioPlaybackThread::Reader(ReaderBase *reader) {
+    void AudioPlaybackThread::Reader(openshot::ReaderBase *reader) {
 		if (source)
 			source->Reader(reader);
 		else {
@@ -103,10 +135,10 @@ namespace openshot
 	}
 
     // Get the current frame object (which is filling the buffer)
-    std::shared_ptr<Frame> AudioPlaybackThread::getFrame()
+    std::shared_ptr<openshot::Frame> AudioPlaybackThread::getFrame()
     {
 	if (source) return source->getFrame();
-	return std::shared_ptr<Frame>();
+	return std::shared_ptr<openshot::Frame>();
     }
 
     // Get the currently playing frame number
@@ -142,7 +174,7 @@ namespace openshot
 
     			// Start new audio device (or get existing one)
     			// Add callback
-				AudioDeviceManagerSingleton::Instance(numChannels)->audioDeviceManager.addAudioCallback(&player);
+				AudioDeviceManagerSingleton::Instance()->audioDeviceManager.addAudioCallback(&player);
 
     			// Create TimeSliceThread for audio buffering
 				time_thread.startThread();
@@ -165,7 +197,7 @@ namespace openshot
     			transport.start();
 
     			while (!threadShouldExit() && transport.isPlaying() && is_playing)
-					usleep(2500);
+					std::this_thread::sleep_for(std::chrono::milliseconds(2));
 
 				// Stop audio and shutdown transport
 				Stop();
@@ -175,7 +207,7 @@ namespace openshot
 				transport.setSource(NULL);
 
 				player.setSource(NULL);
-				AudioDeviceManagerSingleton::Instance(0)->audioDeviceManager.removeAudioCallback(&player);
+				AudioDeviceManagerSingleton::Instance()->audioDeviceManager.removeAudioCallback(&player);
 
 				// Remove source
 				delete source;
